# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: # @Time   : 2023-04-16 8:18
# @Author : 毛鹏

from ..database._mysql_connect import MysqlConnect
from ..database._sqlite_connect import SQLiteConnect

__all__ = [
    'MysqlConnect',
    'SQLiteConnect'
]



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2025-07-04 10:43
# @Author : 毛鹏

from openpyxl import open
import requests
import tempfile
import os
from urllib.parse import urlparse

from mangotools.assertion.text import TextAssertion
from mangotools.decorator import sync_method_callback
from mangotools.models import MethodModel


def open_excel_workbook(file_path, sheet_name=None):
    """
    统一处理Excel文件打开逻辑，支持本地路径和URL下载
    
    Args:
        file_path (str): Excel文件路径或URL
        sheet_name (str, optional): 工作表名称
        
    Returns:
        tuple: (workbook, sheet) 工作簿对象和工作表对象
    """
    # 判断是否为URL
    is_url = file_path.startswith(('http://', 'https://'))
    
    if is_url:
        # 下载文件到临时位置
        response = requests.get(file_path)
        response.raise_for_status()
        
        # 创建临时文件
        temp_file = tempfile.NamedTemporaryFile(delete=False, suffix='.xlsx')
        temp_file.write(response.content)
        temp_file.close()
        
        # 打开Excel文件
        workbook = open(temp_file.name)
        
        # 保存临时文件路径，以便后续清理
        workbook.temp_file_path = temp_file.name
    else:
        # 直接打开本地文件
        workbook = open(file_path)
        workbook.temp_file_path = None
    
    # 处理工作表
    if sheet_name is None or (sheet_name == 'sheet1' and sheet_name not in workbook.sheetnames):
        sheet = workbook.active
    else:
        sheet = workbook[sheet_name] if sheet_name else workbook.active
        
    return workbook, sheet


class ExcelAssertion:
    """Excel文件"""

    @staticmethod
    @sync_method_callback('文件断言', 'Excel断言', 0, [
        MethodModel(n='实际值', f='actual', d=True, v={'文件路径': '请输入文件的绝对路径，或者是缓存的key', '工作表': 'sheet1'}),
        MethodModel(n='预期值', f='expect', d=True, v={'第几行': '请输入要匹配第几行', '行数据': 'json格式的行数据'})
    ])
    def assert_excel_row_data(actual: dict, expect: dict):
        """断言某一行数据"""
        import json
        file_path = actual.get('文件路径')
        sheet_name = actual.get('工作表', 'sheet1')
        row_index = int(expect.get('第几行'))
        row_data = expect.get('行数据')
        if isinstance(row_data, str):
            row_data = json.loads(row_data)
        workbook, sheet = open_excel_workbook(file_path, sheet_name)
        headers = [cell.value for cell in sheet[1]]
        row = [cell.value for cell in sheet[row_index + 1]]
        current_row = dict(zip(headers, row))
        assert current_row == row_data, f"第{row_index}行数据不匹配: 实际={current_row}, 预期={row_data}"
        return f"实际={current_row}, 预期={row_data}"

    @staticmethod
    @sync_method_callback('文件断言', 'Excel断言', 1, [
        MethodModel(n='实际值', f='actual', d=True, v={'文件路径': '请输入文件的绝对路径，或者是缓存的key', '工作表': 'sheet1'}),
        MethodModel(n='预期值', f='expect', d=True, v={'预期行数': '请输入期望行数'})
    ])
    def assert_excel_row_count(actual: dict, expect: dict):
        """断言工作表的行数"""
        file_path = actual.get('文件路径')
        sheet_name = actual.get('工作表', 'sheet1')
        expected_row_count = expect.get('预期行数')
        workbook, sheet = open_excel_workbook(file_path, sheet_name)
        actual_row_count = sheet.max_row
        assert actual_row_count == int(
            expected_row_count), f"行数不匹配, 实际={actual_row_count}, 预期={expected_row_count}"
        return f'实际={actual_row_count}, 预期={expected_row_count}'

    @staticmethod
    @sync_method_callback('文件断言', 'Excel断言', 2, [
        MethodModel(n='实际值', f='actual', d=True, v={'文件路径': '请输入文件的绝对路径，或者是缓存的key', '工作表': 'sheet1'}),
        MethodModel(n='预期值', f='expect', d=True, v={'预期行数': '请输入期望列数'})
    ])
    def assert_excel_column_count(actual: dict, expect: dict):
        """断言工作表的列数"""
        file_path = actual.get('文件路径')
        sheet_name = actual.get('工作表', 'sheet1')
        expected_col_count = expect.get('预期列数')
        workbook, sheet = open_excel_workbook(file_path, sheet_name)
        actual_col_count = sheet.max_column
        assert actual_col_count == expected_col_count, f"列数不匹配, 实际={actual_col_count}, 预期={expected_col_count}"
        return f'实际={actual_col_count}, 预期={expected_col_count}'

    @staticmethod
    @sync_method_callback('文件断言', 'Excel断言', 3, [
        MethodModel(n='实际值', f='actual', d=True, v={'文件路径': '请输入文件的绝对路径，或者是缓存的key', '工作表': 'sheet1'}),
        MethodModel(n='预期值', f='expect', d=True, v={'列字母': '请输入期望列数，示例：A',
                                           '预期值列表': '请输入列表，示例：["任务类型", "301", "302", "303", "304"]'})
    ])
    def assert_excel_column_values(actual: dict, expect: dict):
        """断言某一列的值"""
        file_path = actual.get('文件路径')
        sheet_name = actual.get('工作表', 'sheet1')
        column_letter = expect.get('列字母')
        expected_values = expect.get('预期值列表')
        workbook, sheet = open_excel_workbook(file_path, sheet_name)
        actual_values = [cell.value for cell in sheet[column_letter]]
        assert actual_values == expected_values, f"列{column_letter}的值不匹配, 实际={actual_values}, 预期={expected_values}"
        return f'实际={actual_values}, 预期={expected_values}'

    @staticmethod
    @sync_method_callback('文件断言', 'Excel断言', 4, [
        MethodModel(n='实际值', f='actual', d=True, v={'文件路径': '请输入文件的绝对路径，或者是缓存的key', '工作表': 'sheet1'}),
        MethodModel(n='预期值', f='expect', d=True, v={'预期表头': '请输入列表，示例： ["任务类型", "任务名称"]'})
    ])
    def assert_excel_headers(actual: dict, expect: dict):
        """断言表头"""
        file_path = actual.get('文件路径')
        sheet_name = actual.get('工作表', 'sheet1')
        expected_headers = expect.get('预期表头')
        workbook, sheet = open_excel_workbook(file_path, sheet_name)
        actual_headers = [cell.value for cell in sheet[1]]
        assert actual_headers == expected_headers, f"表头不匹配, 实际={actual_headers}, 预期={expected_headers}"
        return f'实际={actual_headers}, 预期={expected_headers}'

    @staticmethod
    @sync_method_callback('文件断言', 'Excel断言', 5, [
        MethodModel(n='实际值', f='actual', d=True, v={'文件路径': '请输入文件的绝对路径，或者是缓存的key'}),
        MethodModel(n='预期值', f='expect', d=True, v={'工作表列表': '请输入列表，示例：["sheet1", "sheet2"]'})
    ])
    def assert_excel_sheet_names(actual: dict, expect: dict):
        """断言工作表名称列表"""
        file_path = actual.get('文件路径')
        expected_sheet_names = expect.get('工作表列表')
        workbook, _ = open_excel_workbook(file_path)
        actual_sheet_names = workbook.sheetnames
        assert actual_sheet_names == expected_sheet_names, f"工作表名称不匹配, 实际={actual_sheet_names}, 预期={expected_sheet_names}"
        return f'实际={actual_sheet_names}, 预期={expected_sheet_names}'

    @staticmethod
    @sync_method_callback('文件断言', 'Excel断言', 6, [
        MethodModel(n='实际值', f='actual', d=True, v={'文件路径': '请输入文件的绝对路径，或者是缓存的key', '工作表': 'sheet1'}),
        MethodModel(n='预期值', f='expect', d=True, v={'单元格': '请输入单元格，示例：G6', '预期值': '请输入预期值'})
    ])
    def assert_excel_is_equal_to(actual: dict, expect: dict):
        """单元格相等"""
        file_path = actual.get('文件路径')
        sheet_name = actual.get('工作表', 'sheet1')
        cell_address = expect.get('单元格')
        expected_value = expect.get('预期值')
        workbook, sheet = open_excel_workbook(file_path, sheet_name)
        actual_value = sheet[cell_address].value
        try:
            return TextAssertion.p_is_equal_to(float(actual_value), float(expected_value))
        except (ValueError, TypeError, AssertionError):
            pass
        assert actual_value == expected_value or str(actual_value) == str(
            expected_value), f"单元格{cell_address}的值不匹配, 实际={actual_value}, 预期={expected_value}"
        return f'实际={str(actual_value)}, 预期={str(expected_value)}'

    @staticmethod
    @sync_method_callback('文件断言', 'Excel断言', 7, [
        MethodModel(n='实际值', f='actual', d=True, v={'文件路径': '请输入文件的绝对路径，或者是缓存的key', '工作表': 'sheet1'}),
        MethodModel(n='预期值', f='expect', d=True, v={'单元格': '请输入单元格，示例：G6', '预期值': '请输入预期值'})
    ])
    def assert_excel_is_not_equal_to(actual: dict, expect: dict):
        """单元格不相等"""
        file_path = actual.get('文件路径')
        sheet_name = actual.get('工作表', 'sheet1')
        cell_address = expect.get('单元格')
        expected_value = expect.get('预期值')
        workbook, sheet = open_excel_workbook(file_path, sheet_name)
        actual_value = sheet[cell_address].value
        try:
            return TextAssertion.p_is_equal_to(float(actual_value), float(expected_value))
        except (ValueError, TypeError, AssertionError):
            pass
        assert actual_value == expected_value or str(actual_value) == str(
            expected_value), f"单元格{cell_address}的值不匹配, 实际={actual_value}, 预期={expected_value}"
        return f'实际={str(actual_value)}, 预期={str(expected_value)}'

    @staticmethod
    @sync_method_callback('文件断言', 'Excel断言', 8, [
        MethodModel(n='实际值', f='actual', d=True, v={'文件路径': '请输入文件的绝对路径，或者是缓存的key', '工作表': 'sheet1'}),
        MethodModel(n='预期值', f='expect', d=True, v={'单元格': '请输入单元格，示例：G6'})
    ])
    def assert_excel_is_not_none(actual: dict, expect: dict):
        """单元格不是null"""
        file_path = actual.get('文件路径')
        sheet_name = actual.get('工作表', 'sheet1')
        cell_address = expect.get('单元格')
        workbook, sheet = open_excel_workbook(file_path, sheet_name)
        actual_value = sheet[cell_address].value
        return TextAssertion.p_is_not_none(actual_value)

    @staticmethod
    @sync_method_callback('文件断言', 'Excel断言', 9, [
        MethodModel(n='实际值', f='actual', d=True, v={'文件路径': '请输入文件的绝对路径，或者是缓存的key', '工作表': 'sheet1'}),
        MethodModel(n='预期值', f='expect', d=True, v={'单元格': '请输入单元格，示例：G6'})
    ])
    def assert_excel_is_none(actual: dict, expect: dict):
        """单元格是null"""
        file_path = actual.get('文件路径')
        sheet_name = actual.get('工作表', 'sheet1')
        cell_address = expect.get('单元格')
        workbook, sheet = open_excel_workbook(file_path, sheet_name)
        actual_value = sheet[cell_address].value
        return TextAssertion.p_is_none(actual_value)

    @staticmethod
    @sync_method_callback('文件断言', 'Excel断言', 10, [
        MethodModel(n='实际值', f='actual', d=True, v={'文件路径': '请输入文件的绝对路径，或者是缓存的key', '工作表': 'sheet1'}),
        MethodModel(n='预期值', f='expect', d=True, v={'单元格': '请输入单元格，示例：G6', '预期值': '请输入预期值'})
    ])
    def assert_excel_contains(actual: dict, expect: dict):
        """单元格包含预期"""
        file_path = actual.get('文件路径')
        sheet_name = actual.get('工作表', 'sheet1')
        cell_address = expect.get('单元格')
        expected_value = expect.get('预期值')
        workbook, sheet = open_excel_workbook(file_path, sheet_name)
        actual_value = sheet[cell_address].value
        return TextAssertion.p_contains(actual_value, expected_value)

    @staticmethod
    @sync_method_callback('文件断言', 'Excel断言', 11, [
        MethodModel(n='实际值', f='actual', d=True, v={'文件路径': '请输入文件的绝对路径，或者是缓存的key', '工作表': 'sheet1'}),
        MethodModel(n='预期值', f='expect', d=True, v={'单元格': '请输入单元格，示例：G6', '预期值': '请输入预期值'})
    ])
    def assert_excel_not_contain(actual: dict, expect: dict):
        """单元格不包含预期"""
        file_path = actual.get('文件路径')
        sheet_name = actual.get('工作表', 'sheet1')
        cell_address = expect.get('单元格')
        expected_value = expect.get('预期值')
        workbook, sheet = open_excel_workbook(file_path, sheet_name)
        actual_value = sheet[cell_address].value
        return TextAssertion.p_does_not_contain(actual_value, expected_value)


# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description:
# @Time   : 2023-03-07 8:24
# @Author : 毛鹏
import random
from datetime import date, timedelta, datetime

import time
from faker import Faker


class RandomTimeData:
    """ 随机时间类型测试数据 """
    faker = Faker(locale='zh_CN')

    @classmethod
    def time_now_ymdhms(cls, minute=0) -> str:
        """当前年月日时分秒,参数：minute（默认0）"""
        target_time = datetime.now() + timedelta(days=int(minute))
        return target_time.strftime("%Y-%m-%d %H:%M:%S")

    @classmethod
    def time_random_utcnow(cls):
        """当前UTC时间字符串（YYYY-MM-DD HH:MM:SSZ）"""
        return datetime.now().strftime('%Y-%m-%dT%H:%M:%SZ')

    @classmethod
    def time_now_ymd(cls, days=0):
        """当前年月日,参数：days（默认0）"""
        yesterday = datetime.now() - timedelta(days=int(days))
        yesterday_str = yesterday.strftime('%Y-%m-%d')
        return yesterday_str

    @classmethod
    def time_now_hms(cls, minutes: int = 0) -> str:
        """当前时分秒，参数：minutes（默认0），可为正数（未来时间）或负数（过去时间）"""
        target_time = datetime.now() + timedelta(minutes=int(minutes))
        return target_time.strftime('%H:%M:%S')

    @classmethod
    def time_now_ymd_h(cls, hours: int = 0) -> str:
        """当前年月日时，参数：hours（默认0），可为正数（未来时间）或负数（过去时间）"""
        target_time = datetime.now() + timedelta(hours=int(hours))
        return target_time.strftime('%Y-%m-%d %H')

    @classmethod
    def time_now_ymd_hm(cls, minutes: int = 0) -> str:
        """当前年月日时分，参数：minutes（默认0），可为正数（未来时间）或负数（过去时间）"""
        target_time = datetime.now() + timedelta(minutes=int(minutes))
        return target_time.strftime('%Y-%m-%d %H:%M')

    @classmethod
    def time_stamp(cls, minute=0) -> int:
        """几分钟后的时间戳, 参数：minute（默认0）"""
        return int(time.time() + 60 * int(minute)) * 1000

    @classmethod
    def time_next_minute(cls, minute=0) -> str:
        """几分钟后的年月日时分秒 参数：分钟（默认0）"""
        future_time = datetime.now() + timedelta(minutes=int(minute))
        return future_time.strftime('%Y-%m-%d %H:%M:%S')

    @classmethod
    def time_now_year(cls, days: int = 0) -> str:
        """获取当前年份，参数：days（默认0），可为正数（未来时间）或负数（过去时间）"""
        target_time = datetime.now() + timedelta(days=int(days))
        return target_time.strftime('%Y')

    @classmethod
    def time_now_month(cls, days: int = 0) -> str:
        """获取当前月份，参数：days（默认0），可为正数（未来时间）或负数（过去时间）"""
        target_time = datetime.now() + timedelta(days=int(days))
        return target_time.strftime('%m')

    @classmethod
    def time_now_day(cls, days: int = 0) -> str:
        """获取当前日期，参数：days（默认0），可为正数（未来时间）或负数（过去时间）"""
        target_time = datetime.now() + timedelta(days=int(days))
        return target_time.strftime('%d')

    @classmethod
    def time_now_hour(cls, hours: int = 0) -> str:
        """获取当前小时，参数：hours（默认0），可为正数（未来时间）或负数（过去时间）"""
        target_time = datetime.now() + timedelta(hours=int(hours))
        return target_time.strftime('%H')

    @classmethod
    def time_now_minute(cls, minutes: int = 0) -> str:
        """获取当前分钟，参数：minutes（默认0），可为正数（未来时间）或负数（过去时间）"""
        target_time = datetime.now() + timedelta(minutes=int(minutes))
        return target_time.strftime('%M')

    @classmethod
    def time_now_second(cls, seconds: int = 0) -> str:
        """获取当前秒，参数：seconds（默认0），可为正数（未来时间）或负数（过去时间）"""
        target_time = datetime.now() + timedelta(seconds=int(seconds))
        return target_time.strftime('%S')

    @classmethod
    def time_random_year(cls):
        """获取随机年份"""
        return cls.faker.year()

    @classmethod
    def time_random_month(cls):
        """获取随机月份"""
        return cls.faker.month()

    @classmethod
    def time_random_date(cls):
        """获取随机日期"""
        return cls.faker.date()

    @classmethod
    def time_now_int(cls) -> int:
        """获取当前时间戳整形"""
        return int(time.time()) * 1000

    @classmethod
    def time_future_date(cls):
        """未来的随机年月日"""
        return cls.faker.future_date()

    @classmethod
    def time_future_datetime(cls):
        """未来的随机年月日时分秒"""
        return cls.faker.future_datetime()

    @classmethod
    def time_today_date_00(cls):
        """获取今日00:00:00时间"""
        _today = date.today().strftime("%Y-%m-%d") + " 00:00:00"
        return str(_today)

    @classmethod
    def time_today_date_59(cls):
        """获取今日23:59:59时间"""
        _today = date.today().strftime("%Y-%m-%d") + " 23:59:59"
        return str(_today)

    @classmethod
    def time_after_week(cls):
        """获取一周后12点整的时间"""
        _time_after_week = (date.today() + timedelta(days=+6)).strftime("%Y-%m-%d") + " 00:00:00"
        return _time_after_week

    @classmethod
    def time_after_month(cls):
        """获取30天后的12点整时间"""
        _time_after_week = (date.today() + timedelta(days=+30)).strftime("%Y-%m-%d") + " 00:00:00"
        return _time_after_week

    @classmethod
    def time_random_year_str(cls):
        """随机年份字符串"""
        return str(cls.faker.year())

    @classmethod
    def time_random_month_str(cls):
        """随机月份字符串（01-12）"""
        return f"{random.randint(1, 12):02d}"

    @classmethod
    def time_random_day_str(cls):
        """随机日字符串（01-31）"""
        return f"{random.randint(1, 31):02d}"

    @classmethod
    def time_random_hour_str(cls):
        """随机小时字符串（00-23）"""
        return f"{random.randint(0, 23):02d}"

    @classmethod
    def time_random_minute_str(cls):
        """随机分钟字符串（00-59）"""
        return f"{random.randint(0, 59):02d}"

    @classmethod
    def time_random_second_str(cls):
        """随机秒字符串（00-59）"""
        return f"{random.randint(0, 59):02d}"

    @classmethod
    def time_random_ym(cls):
        """随机年月字符串（YYYY-MM）"""
        return f"{cls.faker.year()}-{random.randint(1, 12):02d}"

    @classmethod
    def time_random_ymd_str(cls):
        """随机年月日字符串（YYYY-MM-DD）"""
        return cls.faker.date()

    @classmethod
    def time_random_ymdhm_str(cls):
        """随机年月日时分字符串（YYYY-MM-DD HH:MM）"""
        dt = cls.faker.date_time()
        return dt.strftime('%Y-%m-%d %H:%M')

    @classmethod
    def time_random_ymdhms_str(cls):
        """随机年月日时分秒字符串（YYYY-MM-DD HH:MM:SS）"""
        dt = cls.faker.date_time()
        return dt.strftime('%Y-%m-%d %H:%M:%S')

    @classmethod
    def time_random_hm_str(cls):
        """随机时分字符串（HH:MM）"""
        return f"{random.randint(0, 23):02d}:{random.randint(0, 59):02d}"

    @classmethod
    def time_random_hms_str(cls):
        """随机时分秒字符串（HH:MM:SS）"""
        return f"{random.randint(0, 23):02d}:{random.randint(0, 59):02d}:{random.randint(0, 59):02d}"

    @classmethod
    def time_random_timestamp_s(cls):
        """随机时间戳（秒）"""
        dt = cls.faker.date_time()
        return int(dt.timestamp())

    @classmethod
    def time_random_timestamp_ms(cls):
        """随机时间戳（毫秒）"""
        dt = cls.faker.date_time()
        return int(dt.timestamp() * 1000)

    @classmethod
    def time_random_timestamp_us(cls):
        """随机时间戳（微秒）"""
        dt = cls.faker.date_time()
        return int(dt.timestamp() * 1000000)

    @classmethod
    def time_random_iso8601(cls):
        """随机ISO8601时间字符串"""
        return cls.faker.iso8601()

    @classmethod
    def time_random_rfc3339(cls):
        """随机RFC3339时间字符串"""
        return cls.faker.date_time().isoformat()

    @classmethod
    def time_random_future_ymd(cls):
        """未来随机年月日字符串"""
        return cls.faker.future_date().strftime('%Y-%m-%d')

    @classmethod
    def time_random_past_ymd(cls):
        """过去随机年月日字符串"""
        return cls.faker.past_date().strftime('%Y-%m-%d')

    @classmethod
    def time_random_future_ymdhms(cls):
        """未来随机年月日时分秒字符串"""
        return cls.faker.future_datetime().strftime('%Y-%m-%d %H:%M:%S')

    @classmethod
    def time_random_past_ymdhms(cls):
        """过去随机年月日时分秒字符串"""
        return cls.faker.past_datetime().strftime('%Y-%m-%d %H:%M:%S')

    @classmethod
    def time_random_weekday(cls):
        """随机周几（中文）"""
        weekdays = ["周一", "周二", "周三", "周四", "周五", "周六", "周日"]
        return random.choice(weekdays)

    @classmethod
    def time_random_weekday_num(cls):
        """随机周几（数字1-7）"""
        return random.randint(1, 7)

    @classmethod
    def time_random_quarter(cls):
        """随机季度（Q1-Q4）"""
        return f"Q{random.randint(1, 4)}"

    @classmethod
    def time_random_12h(cls):
        """随机12小时制时间字符串（hh:MM:SS AM/PM）"""
        dt = cls.faker.date_time()
        return dt.strftime('%I:%M:%S %p')

    @classmethod
    def time_random_time_range_str(cls):
        """随机时间区间字符串（如'2023-01-01~2023-01-31'）"""
        start = cls.faker.date_this_year()
        end = cls.faker.future_date(end_date='+30d')
        return f"{start}~{end}"

    @classmethod
    def time_random_utc_offset(cls):
        """随机UTC偏移字符串（如+08:00）"""
        sign = random.choice(['+', '-'])
        hour = random.randint(0, 14)
        minute = random.choice([0, 30, 45])
        return f"{sign}{hour:02d}:{minute:02d}"

    @classmethod
    def time_random_rfc2822(cls):
        """随机RFC2822时间字符串"""
        return cls.faker.date_time().strftime('%a, %d %b %Y %H:%M:%S +0000')

    @classmethod
    def time_random_rfc850(cls):
        """随机RFC850时间字符串"""
        return cls.faker.date_time().strftime('%A, %d-%b-%y %H:%M:%S GMT')

    @classmethod
    def time_random_rfc1123(cls):
        """随机RFC1123时间字符串"""
        return cls.faker.date_time().strftime('%a, %d %b %Y %H:%M:%S GMT')

    @classmethod
    def time_random_rfc1036(cls):
        """随机RFC1036时间字符串"""
        return cls.faker.date_time().strftime('%A, %d-%b-%y %H:%M:%S GMT')

    @classmethod
    def time_random_rfc822(cls):
        """随机RFC822时间字符串"""
        return cls.faker.date_time().strftime('%a, %d %b %y %H:%M:%S +0000')

    @classmethod
    def time_random_rfc3339nano(cls):
        """随机RFC3339纳秒时间字符串"""
        dt = cls.faker.date_time()
        return dt.strftime('%Y-%m-%dT%H:%M:%S.%fZ')

    @classmethod
    def time_cron_time(cls, time_parts) -> str:
        """秒级cron表达式,参数：time_parts"""
        seconds = int(time_parts[0])
        minutes = int(time_parts[1])
        hours = int(time_parts[2])
        current_date = datetime.now().date()
        date_obj = datetime(year=current_date.year,
                            month=current_date.month,
                            day=current_date.day,
                            hour=hours,
                            minute=minutes,
                            second=seconds)

        time_str_result = date_obj.strftime("%H:%M:%S")
        return time_str_result

    @classmethod
    def time_next_minute_cron(cls, minutes=1):
        """按周重复的cron表达式,参数：minutes（默认1）"""
        now = datetime.now() + timedelta(minutes=float(minutes))
        second = f"{now.second:02d}"  # 格式化为两位数
        minute = f"{now.minute:02d}"  # 格式化为两位数
        hour = f"{now.hour:02d}"  # 格式化为两位数
        day = "?"  # 日用问号表示不指定
        month = "*"  # 月用星号表示每个月
        weekday = str(date.today().weekday() + 2)
        return f"{second} {minute} {hour} {day} {month} {weekday}"

    @classmethod
    def time_random_cron(cls):
        """随机cron表达式（分 时 日 月 周）"""
        minute = random.randint(0, 59)
        hour = random.randint(0, 23)
        day = random.randint(1, 28)
        month = random.randint(1, 12)
        week = random.randint(0, 6)
        return f"{minute} {hour} {day} {month} {week}"



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description:
# @Time   : 2022-11-04 22:05
# @Author : 毛鹏

from ..notice._mail_send import EmailSend
from ..notice._wechat_send import WeChatSend
from ..notice._feishu_send import FeiShuSend
__all__ = [
    'WeChatSend',
    'EmailSend',
    'FeiShuSend',
]


# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description:
# @Time   : 2023-08-29 10:23
# @Author : 毛鹏

from cachetools import LRUCache


class CacheTool:
    """ 内存缓存 """

    def __init__(self):
        self._cache: LRUCache = LRUCache(maxsize=10_000)

    def get_cache(self, key: str) -> any:
        """得到缓存key的value"""
        return self._cache.get(key)

    def set_cache(self, key: str, value: any) -> None:
        """设置一个内容到缓存"""
        self._cache[key] = value

    def delete_cache(self, key: str) -> None:
        """删除一个缓存"""
        if key in self._cache:
            del self._cache[key]

    def clear_cache(self) -> None:
        """清理所有缓存"""
        self._cache.clear()

    def has_cache(self, key: str) -> bool:
        """判断缓存是否存在"""
        return key in self._cache

    def get_all(self, ) -> dict:
        """获取全部的缓存数据"""
        return {k: v for k, v in self._cache.items()}

    def set_sql_cache(self, key: str, value: dict) -> None:
        """根据sql进行缓存数据"""
        if key is None or key == '':
            return
        for k, v in zip([part.strip() for part in key.split(',')], value):
            self.set_cache(k, value.get(v))

    def set_list_cache(self, key: list, value: list) -> None:
        """设置list缓存"""
        if key is None or key == '':
            return
        for k, v in zip(key, value):
            self.set_cache(k, v)

    def set_dict_cache(self, data: dict) -> None:
        """设置dict缓存"""
        for k, v in data.items():
            self.set_cache(k, v)



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 飞书通知封装
# @Time   : 2025-12-10 18:30
# @Author : Qwen
import json
from json import JSONDecodeError

import requests

from ..exceptions import MangoToolsError
from ..exceptions.error_msg import ERROR_MSG_0018
from ..models import FeiShuNoticeModel, TestReportModel


class FeiShuSend:
    def __init__(self,
                 notice_config: FeiShuNoticeModel,
                 test_report: TestReportModel | None = None,
                 domain_name: str = None,
                 content: str = None):
        self.notice_config = notice_config
        self.test_report = test_report
        self.domain_name = domain_name
        self.content = content
        self.headers = {"Content-Type": "application/json"}

    def send_feishu_notification(self):
        if self.content is None:
            # 收集所有有值的行
            lines = [
                "**【芒果测试平台测试报告通知】**",
                f"项目产品：<font color='green'>{self.test_report.project_name}/{self.test_report.product_name}</font>",
                f"测试环境：{self.test_report.test_environment}"
            ]

            # 测试套ID行
            if self.test_report.test_suite_id:
                lines.append(f"测试套ID：{self.test_report.test_suite_id}")
            if self.test_report.task_name:
                lines.append(f"任务名称：{self.test_report.task_name}")

            lines.extend([
                "",
                "--------------------",
                "**执行结果**",
                f"<font color='green'>成功率  : {self.test_report.success_rate}%</font>",
                f"执行用例数：<font color='blue'>{self.test_report.case_sum}</font>",
                f"成功用例数：<font color='green'>{self.test_report.success}</font>",
                f"失败用例数：<font color='red'>{self.test_report.fail}个</font>"
            ])

            # 接口相关统计
            if self.test_report.api_case_sum is not None and self.test_report.api_case_sum > 0:
                lines.append(f"接口用例数：{self.test_report.api_case_sum}")
                if self.test_report.api_fail is not None:
                    lines.append(f"接口失败数：{self.test_report.api_fail}")

            # 前端相关统计
            if self.test_report.ui_case_sum is not None and self.test_report.ui_case_sum > 0:
                lines.append(f"前端用例数：{self.test_report.ui_case_sum}")
                if self.test_report.ui_fail is not None:
                    lines.append(f"前端失败数：{self.test_report.ui_fail}")

            # 单元测试相关统计
            if self.test_report.pytest_case_sum is not None and self.test_report.pytest_case_sum > 0:
                lines.append(f"单元用例数：{self.test_report.pytest_case_sum}")
                if self.test_report.pytest_fail is not None:
                    lines.append(f"单元用例失败数：{self.test_report.pytest_fail}")

            # 执行信息
            lines.extend([
                f"用例执行耗时：<font color='orange'>{self.test_report.execution_duration}</font>",
                f"测试开始时间：<font color='gray'>{self.test_report.test_time}</font>",
                "",
                "非相关负责人员可忽略此消息。"
            ])

            # 平台地址
            if self.domain_name:
                lines.append(f"[测试报告入口]({self.domain_name})")

            # 将所有行连接成最终的内容
            content = "\n".join(lines)
        else:
            content = self.content
        self.send_markdown(content)

    def send_markdown(self, content):
        res = requests.post(
            url=self.notice_config.webhook,
            json={
                "msg_type": "post",
                "content": {
                    "post": {
                        "zh_cn": {
                            "title": "芒果测试平台测试报告",
                            "content": [[
                                {
                                    "tag": "text",
                                    "text": content
                                }
                            ]]
                        }
                    }
                }
            },
            headers=self.headers,
            proxies={'http': None, 'https': None}
        )
        try:
            result = res.json()
            if result.get("code") != 0 and result.get("StatusCode") != 0:
                raise MangoToolsError(*ERROR_MSG_0018)
        except JSONDecodeError:
            raise MangoToolsError(*ERROR_MSG_0018)

    def send_text(self, content):
        """
        发送纯文本消息
        
        参数:
            content (str): 文本内容
        """
        res = requests.post(
            url=self.notice_config.webhook,
            json={
                "msg_type": "text",
                "content": {
                    "text": content
                }
            },
            headers=self.headers,
            proxies={'http': None, 'https': None}
        )
        try:
            result = res.json()
            if result.get("code") != 0 and result.get("StatusCode") != 0:
                raise MangoToolsError(*ERROR_MSG_0018)
        except JSONDecodeError:
            raise MangoToolsError(*ERROR_MSG_0018)

    def send_post(self, title, content_list):
        """
        发送富文本消息
        
        参数:
            title (str): 标题
            content_list (list): 内容列表，每个元素是一个包含tag的字典
        """
        res = requests.post(
            url=self.notice_config.webhook,
            json={
                "msg_type": "post",
                "content": {
                    "post": {
                        "zh_cn": {
                            "title": title,
                            "content": [content_list]
                        }
                    }
                }
            },
            headers=self.headers,
            proxies={'http': None, 'https': None}
        )
        try:
            result = res.json()
            if result.get("code") != 0 and result.get("StatusCode") != 0:
                raise MangoToolsError(*ERROR_MSG_0018)
        except JSONDecodeError:
            raise MangoToolsError(*ERROR_MSG_0018)


# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description:
# @Time   : 2023-09-09 23:17
# @Author : 毛鹏
import pymysql
from pymysql.err import InternalError, OperationalError, ProgrammingError
from pymysql.cursors import DictCursor
from datetime import datetime, date

from ..exceptions import MangoToolsError
from ..exceptions.error_msg import ERROR_MSG_0001, ERROR_MSG_0033, ERROR_MSG_0035, ERROR_MSG_0034
from ..models import MysqlConingModel


class MysqlConnect:

    def __init__(self, mysql_config: MysqlConingModel, is_c: bool = True, is_rud: bool = False):
        self.is_c = is_c
        self.is_rud = is_rud
        self.mysql_config = mysql_config
        self.connection = None
        self._connect()

    def _connect(self):
        """建立数据库连接"""
        try:
            self.connection = pymysql.connect(
                host=self.mysql_config.host,
                port=self.mysql_config.port,
                user=self.mysql_config.user,
                password=self.mysql_config.password,
                database=self.mysql_config.database,
                autocommit=True,
                cursorclass=DictCursor,
                charset='utf8mb4',
                connect_timeout=30
            )
        except OperationalError:
            raise MangoToolsError(*ERROR_MSG_0001)
        except InternalError:
            raise MangoToolsError(*ERROR_MSG_0033, value=(self.mysql_config.database,))

    def __enter__(self):
        """支持上下文管理器"""
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """上下文管理器退出时自动关闭连接"""
        self.close()

    def __del__(self):
        """析构函数确保连接关闭"""
        self.close()

    def close(self):
        """安全关闭连接"""
        if self.connection:
            try:
                self.connection.close()
            except Exception:
                # 忽略关闭时的异常
                pass
            finally:
                self.connection = None

    def _convert_datetime_to_string(self, data):
        """递归将日期时间对象转换为字符串"""
        if isinstance(data, dict):
            return {key: self._convert_datetime_to_string(value) for key, value in data.items()}
        elif isinstance(data, list):
            return [self._convert_datetime_to_string(item) for item in data]
        elif isinstance(data, (datetime, date)):
            return data.strftime('%Y-%m-%d %H:%M:%S')
        else:
            return data

    def _execute_query(self, cursor, sql):
        """执行查询语句并处理结果"""
        cursor.execute(sql)
        result = cursor.fetchall()

        # 将日期时间转换为字符串
        if result:
            result = self._convert_datetime_to_string(result)

        return result

    def _execute_update(self, cursor, sql):
        """执行更新语句"""
        cursor.execute(sql)
        # autocommit=True 所以不需要手动commit
        return cursor.rowcount

    def condition_execute(self, sql: str) -> list[dict] | list | int | None:
        """条件执行SQL语句"""
        if not sql or not sql.strip():
            return None

        sql_upper = sql.strip().upper()
        is_select = (sql_upper.startswith(('SELECT', 'WITH', 'SHOW', 'DESC', 'DESCRIBE')) or
                     'SELECT' in sql_upper.split(None, 1)[0])

        if is_select:
            if self.is_c:
                return self.execute(sql)
            else:
                return None
        else:
            if self.is_rud:
                return self.execute(sql)
            else:
                return None

    def execute(self, sql: str) -> list[dict] | int | list:
        """执行SQL语句"""
        if not self.connection or not self.connection.open:
            self._connect()

        with self.connection.cursor() as cursor:
            try:
                sql_upper = sql.strip().upper()
                is_select = (sql_upper.startswith(('SELECT', 'WITH', 'SHOW', 'DESC', 'DESCRIBE')) or
                             'SELECT' in sql_upper.split(None, 1)[0])

                if is_select:
                    result = self._execute_query(cursor, sql)
                    return result
                else:
                    result = self._execute_update(cursor, sql)
                    return result

            except ProgrammingError as e:
                raise MangoToolsError(*ERROR_MSG_0034, value=(sql, str(e)))
            except InternalError as e:
                raise MangoToolsError(*ERROR_MSG_0035, value=(str(e), ))
            except OperationalError as e:
                if "Lost connection" in str(e) or "MySQL server has gone away" in str(e):
                    try:
                        self._connect()
                        return self.execute(sql)
                    except Exception:
                        raise MangoToolsError(*ERROR_MSG_0034, value=(sql, str(e)))
                else:
                    raise MangoToolsError(*ERROR_MSG_0034, value=(sql, str(e)))
            except Exception as e:
                raise MangoToolsError(*ERROR_MSG_0034, value=(sql, str(e)))



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: # @Time   : 2023-08-11 11:12
# @Author : 毛鹏
import hashlib


class EncryptionTool:
    """ 加密 """

    @classmethod
    def md5_32_small(cls, string) -> str:
        """MD5_32位小写加密"""
        md5 = hashlib.md5()
        md5.update(string.encode('utf-8'))
        encrypted_string = md5.hexdigest()
        return encrypted_string

    @classmethod
    def md5_32_large(cls, string) -> str:
        """MD5_32位大写加密"""
        md5 = hashlib.md5()
        md5.update(string.encode('utf-8'))
        encrypted_string = md5.hexdigest().upper()
        return encrypted_string

    @classmethod
    def md5_16_small(cls, string) -> str:
        """MD5_16位小写加密"""
        md5 = hashlib.md5()
        md5.update(string.encode('utf-8'))
        encrypted_string = md5.hexdigest()[8:-8]
        return encrypted_string

    @classmethod
    def md5_16_large(cls, string) -> str:
        """MD5_16位大写加密"""
        md5 = hashlib.md5()
        md5.update(string.encode('utf-8'))
        encrypted_string = md5.hexdigest().upper()[8:-8]
        return encrypted_string



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2025-04-10 21:55
# @Author : 毛鹏
import functools

from mangotools.models import MethodModel

func_info: list[dict] = []
__func_name_list = []


def _func_info_add(_type, class_name, func, parameter, sort):
    class_dict = next((item for item in func_info if item.get("value") == _type), None)
    if not class_dict:
        new_entry = {
            "label": _type,
            "value": _type,
            "children": []
        }
        func_info.append(new_entry)
        class_dict = new_entry
    found_dict = next((item for item in class_dict['children'] if item.get("value") == class_name), None)
    if not found_dict:
        new_entry = {
            "label": class_name,
            "value": class_name,
            "children": []
        }
        class_dict['children'].append(new_entry)
        found_dict = new_entry
    if func.__name__ not in __func_name_list:
        __func_name_list.append(func.__name__)
        found_dict['children'].append({
            "value": func.__name__,
            "label": func.__doc__.split('\n')[-1]  if '\n' in func.__doc__ else func.__doc__,
            "parameter": [i.model_dump() for i in parameter] if parameter else None,
            "sort": sort,
        })


def async_method_callback(_type, class_name, sort=-1, parameter: list[MethodModel] | None = None):
    def decorator(func):
        _func_info_add(_type, class_name, func, parameter, sort)

        @functools.wraps(func)
        async def wrapper(*args, **kwargs):
            return await func(*args, **kwargs)

        return wrapper

    return decorator


def sync_method_callback(_type, class_name, sort=-1, parameter: list[MethodModel] | None = None):
    def decorator(func):
        _func_info_add(_type, class_name, func, parameter, sort)

        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            return func(*args, **kwargs)

        return wrapper

    return decorator



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2025-07-04 14:10
# @Author : 毛鹏
from assertpy import assert_that

from ...decorator import sync_method_callback
from ...models import MethodModel


def _assert_that(actual):
    if actual is None:
        raise AssertionError(f"实际值不能为 None ，可能是在获取实际值的时候就失败了！")
    return assert_that(actual)


class WhatIsEqualToAssertion:
    """值等于什么"""

    @staticmethod
    @sync_method_callback('内容断言', '值等于什么', 0, [
        MethodModel(n='实际值', f='actual', d=True), MethodModel(n='预期值', f='expect', p='请输入断言值', d=True)])
    def p_is_equal_to(actual: str, expect: str):
        """等于expect"""
        try:
            _assert_that(str(actual)).is_equal_to(expect),
        except AssertionError as e:
            raise AssertionError(f'实际={str(actual)}, 预期={expect}') from e
        return f'实际={str(actual)}, 预期={expect}'

    @staticmethod
    @sync_method_callback('内容断言', '值等于什么', 1, [
        MethodModel(n='实际值', f='actual', d=True), MethodModel(n='预期值', f='expect', p='请输入断言值', d=True)])
    def p_is_not_equal_to(actual: str, expect: str):
        """不等于expect"""
        try:
            _assert_that(str(actual)).is_not_equal_to(expect)
        except AssertionError as e:
            raise AssertionError(f'实际={str(actual)}, 预期={expect}') from e
        return f'实际={str(actual)}, 预期={expect}'

    @staticmethod
    @sync_method_callback('内容断言', '值等于什么', 2, [
        MethodModel(n='实际值', f='actual', d=True), MethodModel(n='预期值', f='expect', p='请输入断言值', d=True)])
    def p_is_length(actual: str, expect: str):
        """长度等于expect"""
        try:
            _assert_that(str(actual)).is_length(int(expect))
        except AssertionError as e:
            raise AssertionError(f'实际={str(actual)}, 预期={expect}') from e
        return f'实际={str(actual)}, 预期={expect}'

    @staticmethod
    @sync_method_callback('内容断言', '值等于什么', 3, [
        MethodModel(n='实际值', f='actual', d=True), MethodModel(n='预期值', f='expect', p='请输入断言值', d=True)])
    def p_sum_equal_expect(actual: list, expect: str):
        """长度等于expect"""
        try:
            _assert_that(sum(actual)).is_equal_to(expect)
        except AssertionError as e:
            raise AssertionError(f'实际={str(actual)}, 预期={expect}') from e
        return f'实际={str(actual)}, 预期={expect}'



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 错误消息统一管理
# @Time   : 2024-02-01 10:00
# @Author : 毛鹏
ERROR_MSG_0001 = (3001, 'mysql链接错误，请检查服务器是否可以链接到这个地址，或是IP端口账号密码错误')
ERROR_MSG_0002 = (3002, '无法获取缓存的key【{}】的值，请检查是否设置这个key')
ERROR_MSG_0003 = (3003, 'jsonpath表达式错误或响应中不包含取的值，jsonpath：【{}】')
ERROR_MSG_0004 = (3004, '数据不可以被转换成json')
ERROR_MSG_0006 = (3006, '随机字符串函数只支持传入int类型数字')
ERROR_MSG_0033 = (3033, 'mysql中无：【{}】库')
ERROR_MSG_0034 = (3034, 'sql存在语法错误，请检查sql语句：{}，错误信息：{}')
ERROR_MSG_0035 = (3035, 'sql语句是空，或查询字段或条件字段不存在')
ERROR_MSG_0047 = (3047, '随机数据方法不存在，请检查输入的方法名称是否正确，方法名称：{}，错误类型：{}')
ERROR_MSG_0061 = (3061, '前置或者后置的自定义函数没有按照文档规范填写，或者语法不正确，请参照文档后重新填写')
ERROR_MSG_0014 = (3014, '企微发送失败，请检查企微发送配置')
ERROR_MSG_0015 = (3015, '的类型错误，请联系管理员进行排查问题')
ERROR_MSG_0016 = (3016, '邮件发送失败，请检查邮件的配置')
ERROR_MSG_0017 = (3017, '断言失败，提示： 【{}】')
ERROR_MSG_0018 = (3018, 'webhook的链接错误！')
ERROR_MSG_0019 = (3019, '文件断言必须要使用可以被序列化为json格式数据')
ERROR_MSG_0020 = (3020, 'SQL断言的期望值必须要使用可以被序列化为json格式数据')
ERROR_MSG_0021 = (3021, 'SQL断言必须要传入SQL链接对象')
ERROR_MSG_0022 = (3022, '请根据帮助文档，传入函数需要的参数个数，不要多传或少传')
ERROR_MSG_0023 = (3023, '传入的函数不合法，请参考帮助文档')
ERROR_MSG_0024 = (3024, '提交到git仓库时，未提供有效的身份信息')



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2025-11-21 14:17
# @Author : 毛鹏
import inspect
import json
import sys
from typing import List, Dict, Any
from mangotools.data_processor import DataProcessor

# 确保标准输出使用 UTF-8
if hasattr(sys.stdout, 'reconfigure'):
    sys.stdout.reconfigure(encoding='utf-8')


def safe_string_conversion(text):
    """安全处理字符串编码"""
    if text is None:
        return None

    if isinstance(text, bytes):
        try:
            return text.decode('utf-8')
        except UnicodeDecodeError:
            try:
                return text.decode('gbk')
            except:
                return "编码错误"

    # 确保是有效的 Unicode 字符串
    try:
        return str(text)
    except:
        return "字符串转换错误"


def get_function_info(func) -> Dict[str, Any]:
    """获取函数的详细信息"""
    docstring = func.__doc__
    if docstring:
        # 清理文档字符串
        docstring = docstring.strip()

    info = {
        'name': func.__name__,
        'docstring': safe_string_conversion(docstring),
        'signature': str(inspect.signature(func)),
        'qualname': getattr(func, '__qualname__', func.__name__),
    }
    return info


def get_class_methods_info(cls) -> List[Dict[str, Any]]:
    """获取类的所有方法信息"""
    methods_list = []

    for name in dir(cls):
        if name.startswith('__') and name.endswith('__'):
            continue

        obj = getattr(cls, name)

        if callable(obj) and not isinstance(obj, type):
            try:
                func_info = get_function_info(obj)
                methods_list.append(func_info)
            except Exception as e:
                print(f"跳过 {name}: {e}")

    return methods_list


if __name__ == "__main__":
    class_methods = get_class_methods_info(DataProcessor)

    # 确保使用 UTF-8 编码写入文件
    with open('test_data.json', 'w', encoding='utf-8') as f:
        json.dump(class_methods, f, ensure_ascii=False, indent=4)

    # 控制台输出也确保编码正确
    for method in class_methods:
        print(f"\n方法: {method['name']}")
        docstring = method['docstring'] or "无文档"
        print(f"  文档: {docstring}")
        print(f"  签名: {method['signature']}")


# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2025-04-10 23:08
# @Author : 毛鹏
import functools
from typing import Type, Callable, Any


def inject_to_class(target_class: Type[Any]):
    """装饰器工厂，用于将函数注入到指定类中"""

    def decorator(func: Callable) -> Callable:
        """实际装饰器"""
        setattr(target_class, func.__name__, func)

        @functools.wraps(func)
        async def wrapper(*args, **kwargs):
            return await func(*args, **kwargs)

        return wrapper

    return decorator



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2025-04-30 17:12
# @Author : 毛鹏
import json
import tempfile
import unittest

from mangotools.data_processor import SqlCache
from mangotools.enums import CacheValueTypeEnum


class TestSqlCache(unittest.TestCase):
    def setUp(self):
        """在每个测试方法前初始化"""
        temp_db = tempfile.NamedTemporaryFile(delete=False)
        temp_db.close()
        db_path = temp_db.name
        self.cache = SqlCache(db_path)

    def test_01(self):
        self.cache.set_sql_cache("test_str", "hello world", CacheValueTypeEnum.STR)
        print(type(self.cache.get_sql_cache("test_str")), self.cache.get_sql_cache("test_str"))
        assert self.cache.get_sql_cache("test_str") == "hello world"

    def test_02(self):
        self.cache.set_sql_cache("test_int", 42, CacheValueTypeEnum.INT)
        print(type(self.cache.get_sql_cache("test_int")), self.cache.get_sql_cache("test_int"))
        assert self.cache.get_sql_cache("test_int") == 42

    def test_03(self):
        self.cache.set_sql_cache("test_float", 3.14, CacheValueTypeEnum.FLOAT)
        print(type(self.cache.get_sql_cache("test_float")), self.cache.get_sql_cache("test_float"))
        assert self.cache.get_sql_cache("test_float") == 3.14

    def test_04(self):
        self.cache.set_sql_cache("test_bool", True, CacheValueTypeEnum.BOOL)
        print(type(self.cache.get_sql_cache("test_bool")), self.cache.get_sql_cache("test_bool"))
        assert self.cache.get_sql_cache("test_bool") is True

    def test_05(self):
        self.cache.set_sql_cache("test_none", None, CacheValueTypeEnum.NONE)
        print(type(self.cache.get_sql_cache("test_none")), self.cache.get_sql_cache("test_none"))
        assert self.cache.get_sql_cache("test_none") is None

    def test_06(self):
        test_list = [1, 2, 3, "four"]
        self.cache.set_sql_cache("test_list", test_list, CacheValueTypeEnum.LIST)
        print(type(self.cache.get_sql_cache("test_list")), self.cache.get_sql_cache("test_list"))
        assert self.cache.get_sql_cache("test_list") == test_list

    def test_07(self):
        test_dict = {"a": 1, "b": 2, "c": "three"}
        self.cache.set_sql_cache("test_dict", test_dict, CacheValueTypeEnum.DICT)
        print(type(self.cache.get_sql_cache("test_dict")), self.cache.get_sql_cache("test_dict"))
        assert self.cache.get_sql_cache("test_dict") == test_dict

    def test_08(self):
        test_tuple = (1, 2, 3, "four")
        self.cache.set_sql_cache("test_tuple", test_tuple, CacheValueTypeEnum.TUPLE)
        print(type(self.cache.get_sql_cache("test_tuple")), self.cache.get_sql_cache("test_tuple"))
        assert self.cache.get_sql_cache("test_tuple") == test_tuple

    def test_09(self):
        test_json = {"name": "John", "age": 30, "city": "New York"}
        self.cache.set_sql_cache("test_json", test_json, CacheValueTypeEnum.JSON)
        print(type(self.cache.get_sql_cache("test_json")), self.cache.get_sql_cache("test_json"))
        assert self.cache.get_sql_cache("test_json") == json.dumps(test_json)

    def test_10(self):
        print(json.dumps(self.cache.get_sql_all()))
        assert self.cache.contains_sql_cache("nonexistent") is False

    def test_11(self):
        print("测试删除。..")
        self.cache.delete_sql_cache("test_str")
        assert self.cache.get_sql_cache("test_str") is None
        assert self.cache.contains_sql_cache("test_str") is False

    def test_12(self):
        print("测试清除。..")
        self.cache.clear_sql_cache()
        assert self.cache.get_sql_cache("test_int") is None
        assert self.cache.contains_sql_cache("test_int") is False



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2025-01-21 11:42
# @Author : 毛鹏
import unittest

from mangotools.data_processor import DataProcessor


class TestCache(unittest.TestCase):

    def test_001(self):
        """设置缓存和使用华城"""
        value = '设置到缓存中'
        processor = DataProcessor()
        processor.set_cache('value', value)  # 首先我们把value设置一个值
        assert processor.get_cache('value') == value
        return processor.get_cache('value')

    def test_002(self):
        """替换${{}}中间的内容"""
        key = '芒果测试平台'
        value = '替换：${{key}}'
        processor = DataProcessor()
        processor.set_cache('key', key)  # 首先我们把value设置一个值
        assert processor.replace(value) == '替换：芒果测试平台'
        return processor.replace(value)  # 调用replace进行替换

    def test_003(self):
        """获取公共方法中的数据"""
        processor = DataProcessor()
        assert processor.replace('${{md5_32_small(123456)}}') is not None
        return processor.replace('${{md5_32_small(123456)}}')

    def test_004(self):
        """直接将获取到的内容存到缓存中"""
        str_ = "我是基于时间戳的5位随机数：${{number_time_5()|flow名称}}"
        processor = DataProcessor()
        value = processor.replace(str_)
        assert value == f"我是基于时间戳的5位随机数：{processor.get_cache('flow名称')}"
        return f"我是基于时间戳的5位随机数：{processor.get_cache('flow名称')}"

    def test_005(self):
        """获取公共方法中的数据"""
        processor = DataProcessor()
        processor.set_cache('key', '123456')
        assert processor.get_cache('key') == '123456'
        assert processor.replace('${{md5_32_small(${{key}})}}') == 'e10adc3949ba59abbe56e057f20f883e'

    def test_006(self):
        processor = DataProcessor()
        processor.set_cache('key', '123456')
        assert processor.get_cache('key') == '123456'
        assert processor.replace('${{md5_32_small(string=${{key}})}}') == 'e10adc3949ba59abbe56e057f20f883e'

    def test_007(self):
        processor = DataProcessor()
        processor.set_cache('key', '123456')
        assert processor.get_cache('key') == '123456'
        assert processor.replace('${{md5_32_small(${{key}})}}') == 'e10adc3949ba59abbe56e057f20f883e'

    def test_008(self):
        processor = DataProcessor()
        processor.set_cache('key', '123456')
        assert processor.get_cache('key') == '123456'
        assert processor.replace('${{md5_32_small(${{key}},)}}') == 'e10adc3949ba59abbe56e057f20f883e'



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description:
# @Time   : 2022-11-04 22:05
# @Author : 毛鹏
import logging
import os

import colorlog
from concurrent_log_handler import ConcurrentRotatingFileHandler


class LogHandler:
    level_relations = {
        'debug': logging.DEBUG,
        'info': logging.INFO,
        'warning': logging.WARNING,
        'error': logging.ERROR,
        'critical': logging.CRITICAL
    }

    def __init__(self, file_name: str, level: str):
        # 确保日志目录存在
        log_dir = os.path.dirname(file_name)
        if log_dir and not os.path.exists(log_dir):
            os.makedirs(log_dir, exist_ok=True)

        self.logger = logging.getLogger(file_name)
        self.logger.setLevel(self.level_relations.get(level, logging.DEBUG))

        self._add_console_handler(level)

        self._add_file_handler(file_name, level)

    def _add_console_handler(self, level):
        """添加彩色控制台日志处理器"""
        console_handler = logging.StreamHandler()
        console_handler.setFormatter(self._get_color_formatter(level))
        self.logger.addHandler(console_handler)

    def _add_file_handler(self, filename, level):
        """添加多进程安全的文件日志处理器"""
        file_handler = ConcurrentRotatingFileHandler(
            filename,
            mode='a',
            maxBytes=100 * 1024 * 1024,  # 100MB（按大小轮转作为备用策略）
            backupCount=3,
            encoding='utf-8',
            use_gzip=False
        )

        file_fmt = "%(levelname)-8s[%(asctime)s][%(filename)s:%(lineno)d] %(message)s"
        file_handler.setFormatter(logging.Formatter(file_fmt))
        self.logger.addHandler(file_handler)

    @staticmethod
    def _get_color_formatter(level):
        """获取彩色日志格式"""
        if level in ["debug", "info"]:
            fmt = "%(log_color)s[%(asctime)s] [%(levelname)s]: %(message)s"
        else:
            fmt = "%(log_color)s[%(asctime)s] [%(filename)s-->行:%(lineno)d] [%(levelname)s]: %(message)s"

        return colorlog.ColoredFormatter(
            fmt,
            log_colors={
                'DEBUG': 'cyan',
                'INFO': 'green',
                'WARNING': 'yellow',
                'ERROR': 'red',
                'CRITICAL': 'purple',
            }
        )



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2025-07-07 16:38
# @Author : 毛鹏
import unittest

from mangotools.assertion import MangoAssertion
from mangotools.database import MysqlConnect
from mangotools.models import MysqlConingModel


class TestAssSql(unittest.TestCase):

    def test_001(self):
        mysql_conn = MysqlConnect(MysqlConingModel(
            **{"host": "172.16.100.26", "port": 3306, "user": "root", "password": "Root@123",
               "database": "z_desk_efficiency_pre"}
        ))

        MangoAssertion(mysql_conn).ass("assert_sql_is_not_empty",
                                       "SELECT oss_url FROM `data_subscription_mine_task_send_detail` WHERE subscription_mine_id = '387';")



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2024-09-23 16:55
# @Author : 毛鹏
import unittest

from mangotools.data_processor import DataProcessor
from mangotools.database import MysqlConnect
from mangotools.log_collector import set_log
from mangotools.mangos import Mango
from mangotools.models import MysqlConingModel


class TestMango(unittest.TestCase):

    def test_001(self, is_send=False):
        if is_send:
            text = '哈哈哈，测试内容！'
            Mango.s(self.test_001, text, self.test_001)

    def test_004(self):
        mysql_connect = MysqlConnect(MysqlConingModel(
            host='118.196.24.189',
            port=3306,
            user='root',
            password='mP123456&',
            database='mango_server',
        ))
        result = mysql_connect.execute('SHOW TABLES;')
        assert result is not None

    def test_005(self):
        pass

    def test_006(self):
        value = 'haha'
        key = '${{key}}'
        processor = DataProcessor()
        Mango.s_e(processor, 'set_cache', {'key': 'key', "value": value})
        print(processor.replace(key))
        assert Mango.s_e(processor, 'replace', key) == value

    def test_0066(self):
        key = '${{randint(left= 1,left=2)}}'
        processor = DataProcessor()
        print(processor.replace(key))

    def test_007(self):
        log = set_log("D:\GitCode\MangoKit\logs", True)
        log.debug('DEBUG')
        log.info("INFO")
        log.warning("WARNING")
        log.error("ERROR")
        log.critical("CRITICAL")

    def test_008(self):
        pass



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2025-07-04 11:13
# @Author : 毛鹏
import json
import unittest

from mangotools.assertion import MangoAssertion, FileAssertion

FILE_PATH = r'D:\code\mango_tools\tests\任务爬取逻辑.xlsx'
SHEET_NAME = 'Sheet1'


class TestExcelTaskSheet(unittest.TestCase):
    def test_excel_headers(self):
        actual = {'文件路径': FILE_PATH, '工作表': SHEET_NAME}
        expect = {'预期表头': [
            '任务类型', '任务名称', '重试次数', '卡主任务重置状态的间隔时间',
            '去重间隔时间', '去重时间间隔（爬虫）', '自动触发类型']}
        MangoAssertion().ass(FileAssertion.assert_excel_headers.__name__, actual, expect)

    def test_excel_all_row_data(self):
        all_rows = [
            {'任务类型': 301, '任务名称': '小红书关键词', '重试次数': '1+3', '卡主任务重置状态的间隔时间': '10（分钟）',
             '去重间隔时间': '-', '去重时间间隔（爬虫）': '-', '自动触发类型': '自动触发类型1'},
            {'任务类型': 302, '任务名称': '小红书聚光创意报表', '重试次数': '1+3',
             '卡主任务重置状态的间隔时间': '30（分钟）', '去重间隔时间': '-', '去重时间间隔（爬虫）': '-',
             '自动触发类型': '自动触发类型2'},
            {'任务类型': 303, '任务名称': '小红书聚光笔记报表', '重试次数': '1+3',
             '卡主任务重置状态的间隔时间': '30（分钟）', '去重间隔时间': '-', '去重时间间隔（爬虫）': '-',
             '自动触发类型': '自动触发类型3'},
            {'任务类型': 304, '任务名称': '小红书聚光关键词报表', '重试次数': '1+3',
             '卡主任务重置状态的间隔时间': '30（分钟）', '去重间隔时间': '-', '去重时间间隔（爬虫）': '-',
             '自动触发类型': '自动触发类型4'},
            {'任务类型': 305, '任务名称': '小红书乘风创意报表', '重试次数': '1+3',
             '卡主任务重置状态的间隔时间': '30（分钟）', '去重间隔时间': '-', '去重时间间隔（爬虫）': '-',
             '自动触发类型': '自动触发类型5'},
            {'任务类型': 401, '任务名称': '小红书笔记详情（数据订阅）', '重试次数': '1+3',
             '卡主任务重置状态的间隔时间': '15（分钟）', '去重间隔时间': '-', '去重时间间隔（爬虫）': '2（小时）',
             '自动触发类型': '自动触发类型6'},
            {'任务类型': 402, '任务名称': '小红书笔记评论（数据订阅）', '重试次数': '1+3',
             '卡主任务重置状态的间隔时间': '15（分钟）', '去重间隔时间': '-', '去重时间间隔（爬虫）': '2（小时）',
             '自动触发类型': '自动触发类型7'},
            {'任务类型': 401, '任务名称': '小红书笔记详情（项目笔记）', '重试次数': '1+3',
             '卡主任务重置状态的间隔时间': '15（分钟）', '去重间隔时间': '-', '去重时间间隔（爬虫）': '2（小时）',
             '自动触发类型': '自动触发类型8'},
            {'任务类型': 402, '任务名称': '小红书笔记评论（项目笔记）', '重试次数': '1+3',
             '卡主任务重置状态的间隔时间': '15（分钟）', '去重间隔时间': '-', '去重时间间隔（爬虫）': '2（小时）',
             '自动触发类型': '自动触发类型9'},
            {'任务类型': 601, '任务名称': '蒲公英代下单截图', '重试次数': '-', '卡主任务重置状态的间隔时间': '-',
             '去重间隔时间': '-', '去重时间间隔（爬虫）': '-', '自动触发类型': '自动触发类型10'}
        ]
        for index, row_data in enumerate(all_rows):
            actual = {'文件路径': FILE_PATH, '工作表': SHEET_NAME}
            expect = {'第几行': f'{1 + index}', '行数据': row_data}
            MangoAssertion().ass(FileAssertion.assert_excel_row_data.__name__, actual, expect)

    def test_excel_cell_value(self):
        actual = {'文件路径': FILE_PATH, '工作表': SHEET_NAME}
        expect = {'单元格': 'B2', '预期值': '小红书关键词'}
        MangoAssertion().ass(FileAssertion.assert_excel_is_equal_to.__name__, actual, expect)
        expect = {'单元格': 'G6', '预期值': '自动触发类型5'}
        MangoAssertion().ass(FileAssertion.assert_excel_is_equal_to.__name__, actual, expect)

    def test_excel_row_count(self):
        actual = {'文件路径': FILE_PATH, '工作表': SHEET_NAME}
        expect = {'预期行数': '11'}  # 1表头+10数据行
        MangoAssertion().ass(FileAssertion.assert_excel_row_count.__name__, actual, expect)

    def test_excel_column_count(self):
        actual = {'文件路径': FILE_PATH, '工作表': SHEET_NAME}
        expect = {'预期列数': 7}
        MangoAssertion().ass(FileAssertion.assert_excel_column_count.__name__, actual, expect)

    def test_excel_sheet_names(self):
        actual = json.dumps({'文件路径': FILE_PATH})
        expect = {'工作表列表': [SHEET_NAME, 'Sheet2', 'Sheet3']}
        MangoAssertion().ass(FileAssertion.assert_excel_sheet_names.__name__, actual, expect)

    def test_excel_column_values(self):
        actual = {'文件路径': FILE_PATH, '工作表': SHEET_NAME}
        expect = {'列字母': 'A', '预期值列表': [
            '任务类型', 301, 302, 303, 304, 305, 401, 402, 401, 402, 601
        ]}
        MangoAssertion().ass(FileAssertion.assert_excel_column_values.__name__, actual, expect)

    def test_excel_column_values1(self):
        actual = {'文件路径': r"D:\code\mango_tools\tests\任务爬取逻辑.xlsx",
                  '工作表': SHEET_NAME}
        expect = {'单元格': 'A2', '预期值': '301'}
        print(MangoAssertion().ass(FileAssertion.assert_excel_is_equal_to.__name__, actual, expect))



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2025-07-04 11:13
# @Author : 毛鹏
import os
import shutil
import unittest

from dulwich.repo import Repo

from mangotools.mangos import GitRepoOperator


class Log():
    def info(self, msg):
        print(msg)

    def debug(self, msg):
        print(msg)

    def error(self, msg):
        print(msg)

    def warning(self, msg):
        print(msg)

    def critical(self, msg):
        print(msg)


log = Log()


class TestGitOperations(unittest.TestCase):
    REPO_URL = "https://gitee.com/mao-peng/MangoPytest.git"
    path = 'D:\code\mango_tools'
    TEST_REPO_DIR = os.path.join(path, "mango_pytest")
    USERNAME = 'mao-peng'
    PASSWORD = 'mP729164035'

    @classmethod
    def setUpClass(cls):
        """创建测试用的GitRepoOperator实例"""
        cls.git_operator = GitRepoOperator(
            cls.REPO_URL,
            cls.path,
            log,
            username=cls.USERNAME,
            password=cls.PASSWORD,
        )
        try:
            # 确保测试目录不存在
            if os.path.exists(cls.TEST_REPO_DIR):
                shutil.rmtree(cls.TEST_REPO_DIR)
        except PermissionError as e:
            pass

    @classmethod
    def tearDownClass(cls):
        """清理测试创建的目录"""
        pass

    def test_01_clone_repository(self):
        """测试仓库克隆功能"""
        self.git_operator.clone(force_clone=False)

        self.assertTrue(os.path.exists(self.TEST_REPO_DIR))
        self.assertTrue(os.path.exists(os.path.join(self.TEST_REPO_DIR, '.git')))

        repo = Repo(self.TEST_REPO_DIR)
        config = repo.get_config()
        remote_url = config.get(('remote', 'origin'), 'url')
        print(remote_url)

    def test_02_pull_updates(self):
        """测试拉取更新功能"""
        if not os.path.exists(self.TEST_REPO_DIR):
            self.git_operator.clone()
        self.git_operator.pull(accept_remote=True)

        repo_info = self.git_operator.get_repo_info()
        print(repo_info)

    def test_03_push_changes(self):
        """测试推送更改功能"""
        test_file = os.path.join(self.TEST_REPO_DIR, "README.md")
        self.git_operator.clone()
        if not os.path.exists(test_file):
            with open(test_file, 'w') as f:
                f.write("This is a test file for Git operations\n")
        else:
            with open(test_file, 'a') as f:
                f.write("1\n")

        self.git_operator.push()

        repo_info = self.git_operator.get_repo_info()
        print(repo_info['is_dirty'])

        with open(test_file, 'r', encoding='utf-8') as f:
            content = f.read()
            self.assertIn("1", content)

    def test_04_get_repo_info(self):
        """测试获取仓库信息功能"""
        repo_info = self.git_operator.get_repo_info()

        # 验证返回的信息结构
        self.assertIsInstance(repo_info, dict)
        self.assertIn('active_branch', repo_info)
        self.assertIn('commit_hash', repo_info)
        self.assertIn('is_dirty', repo_info)
        self.assertIn('remote_url', repo_info)

        # 打印仓库信息（与示例中的格式一致）
        print("\n📊 仓库信息:")
        print(f"   当前分支: {repo_info['active_branch']}")
        print(f"   最新提交: {repo_info['commit_hash']}")
        print(f"   是否有未提交更改: {'是' if repo_info['is_dirty'] else '否'}")
        print(f"   远程仓库: {repo_info['remote_url']}")


if __name__ == '__main__':
    unittest.main(verbosity=2)
